--- grx246p1/test/fontdemo.c	2002-08-27 12:27:34.000000000 +0300
+++ grx246/test/fontdemo.c	2003-02-24 21:37:44.000000000 +0200
@@ -69,6 +69,23 @@
 	curx = cury = 0;
 }
 
+static void loadfont(char *name)
+{
+	char *testname;
+
+	opt.txo_font = GrLoadFont(name);
+	if(opt.txo_font == NULL && (testname = malloc(strlen(name) + 10)) != NULL) {
+	    /* try again, this is a test so we have a well known path */
+#if defined(__MSDOS__) || defined(__WIN32__)
+	    sprintf( testname,"..\\fonts\\%s",name );
+#else
+	    sprintf( testname,"../fonts/%s",name );
+#endif
+	    opt.txo_font = GrLoadFont(testname);
+	    free(testname);
+	}
+}
+
 int GRXMain(int argc, char **argv)
 {
 	int i, n;
@@ -78,8 +95,7 @@
 
 	int width = 0, height = 0, bpp = 8, gray = 192, attributes = 0;
 
-	char *name, *testname;
-	GrFontHeader *hdr;
+	char *name;
 	FILE *f;
 	char buffer[0x20];
 	GrKeyType key;
@@ -128,39 +144,38 @@
 	}
 
 	name = argv[i++];
-	opt.txo_font = GrLoadFont(name);
-        if(opt.txo_font == NULL && (testname = malloc(strlen(name) + 10)) != NULL) {
-            /* try again, this is a test and the path can not been set yet */
-#if defined(__MSDOS__) || defined(__WIN32__)
-            sprintf( testname,"..\\fonts\\%s",name );
-#else
-            sprintf( testname,"../fonts/%s",name );
-#endif
-            opt.txo_font = GrLoadFont(testname);
-	    free(testname);
-        }
-	if(opt.txo_font == NULL) {
-	    fprintf(stderr, "%s: load error\n", name);
-	    return(1);
-	}
-	hdr = &opt.txo_font->h;
+	/* try loadfont() before GrSetMode() to get font height - may fail */
+	loadfont(name);
 
 	if(height == 0) {
 	    if(width == 0) {
-		switch(hdr->height) {
-		    case 8 :
-		    case 14 : height = 400; break;
-		    case 16 : height = 480; break;
-		    case 20 : height = 600; break;
-		    case 24 : height = 768; break;
-		    default : height = hdr->height < 20 ? 480 : 768;
+		if(opt.txo_font != NULL) {
+		    switch(opt.txo_font->h.height) {
+			case 8 :
+			case 14 : height = 400; break;
+			case 16 : height = 480; break;
+			case 20 : height = 600; break;
+			case 24 : height = 768; break;
+			default : height = opt.txo_font->h.height < 20 ? 480 : 768;
+		    }
 		}
+		else height = 600;
 	    }
 	    else height = width * 3 / 4;
 	}
 	if(width == 0) width = height == 400 ? 640 : height * 4 / 3;
 
 	GrSetMode(GR_width_height_bpp_graphics, width, height, bpp);
+	if(opt.txo_font == NULL)
+	{
+	    loadfont(name);
+	    /* a loadfont() failure after GrSetMode() is 100% a load error */
+	    if(opt.txo_font == NULL) {
+		fprintf(stderr, "%s: load error\n", name);
+		return(1);
+	    }
+	}
+
 	if(!gray || (opt.txo_fgcolor.v = GrAllocColor(gray, gray, gray)) == GrNOCOLOR) opt.txo_fgcolor.v = GrWhite();
 	if(attributes & 0x02) opt.txo_fgcolor.v |= GR_UNDERLINE_TEXT;
 	opt.txo_bgcolor.v = GrBlack();
@@ -170,7 +185,7 @@
 	opt.txo_xalign = GR_ALIGN_LEFT;
 	opt.txo_yalign = GR_ALIGN_TOP;
 
-	sprintf(buffer, "%s %dx%d", hdr->name, GrCharWidth('A', &opt), GrCharHeight('A', &opt));
+	sprintf(buffer, "%s %dx%d", opt.txo_font->h.name, GrCharWidth('A', &opt), GrCharHeight('A', &opt));
 	gputs(buffer);
 	sprintf(buffer, "%dx%d@%lu", GrSizeX(), GrSizeY(), (unsigned long) GrNumColors());
 	gputs(buffer);
@@ -180,7 +195,7 @@
 	gputs("the quick brown fox jumps over the lazy dog");
 	gnewl();
 
-	if(hdr->minchar <= 0xC0 && hdr->minchar + hdr->numchars >= 0x100) {
+	if(opt.txo_font->h.minchar <= 0xC0 && opt.txo_font->h.minchar + opt.txo_font->h.numchars >= 0x100) {
 	    gputs("     ");
 	    gputs("     ");
 	    gnewl();
@@ -188,8 +203,8 @@
 
 	/* ascii table, or to be precise, a full table of the current font */
 	opt.txo_chrtype = GR_WORD_TEXT;
-	for(c = 0; c < hdr->numchars; c++) {
-	    gputc(hdr->minchar + c);
+	for(c = 0; c < opt.txo_font->h.numchars; c++) {
+	    gputc(opt.txo_font->h.minchar + c);
 	    if(c % 0x20 == 0x1F) gnewl();
 	}
 	gnewl();
